# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------------
# Copyright © 2021, Francisco Palm
#
# Licensed under the terms of the MIT license
# ----------------------------------------------------------------------------
"""
Spyder Pomodoro Timer Main Container.
"""

# Third party imports
import qtawesome as qta
from qtpy.QtWidgets import QToolButton

# Spyder imports
from spyder.api.config.decorators import on_conf_change
from spyder.api.translations import get_translation
from spyder.api.widgets.main_container import PluginMainContainer
from spyder.utils.icon_manager import ima

# Local imports
from spyder_pomodoro_timer.spyder.widgets import (
    PomodoroTimerStatus,
    PomodoroTimerToolbar,
    INTERVAL
)
from spyder_pomodoro_timer.spyder.api import (
    PomodoroToolbarActions,
    PomodoroToolbarSections,
    PomodoroMenuSections,
)

_ = get_translation("spyder_pomodoro_timer.spyder")


class SpyderPomodoroTimerContainer(PluginMainContainer):

    # Signals

    # --- PluginMainContainer API
    # ------------------------------------------------------------------------
    def setup(self):
        # Widgets
        self.pomodoro_timer_status = PomodoroTimerStatus(self)
        title = _("Pomodoro Timer Toolbar")
        self.pomodoro_timer_toolbar = PomodoroTimerToolbar(self, title)

        # Actions
        start_timer_action = self.create_action(
            PomodoroToolbarActions.Start,
            text=_("Start"),
            tip=_("Start timer"),
            icon=qta.icon("fa.play-circle", color=ima.MAIN_FG_COLOR),
            triggered=self.start_pomodoro_timer,
        )

        pause_timer_action = self.create_action(
            PomodoroToolbarActions.Pause,
            text=_("Pause"),
            tip=_("Pause timer"),
            icon=qta.icon("fa.pause-circle", color=ima.MAIN_FG_COLOR),
            triggered=self.pause_pomodoro_timer,
        )

        stop_timer_action = self.create_action(
            PomodoroToolbarActions.Stop,
            text=_("Stop"),
            tip=_("Stop timer"),
            icon=qta.icon("fa.stop-circle", color=ima.MAIN_FG_COLOR),
            triggered=self.stop_pomodoro_timer,
        )

        self.pomodoro_menu = self.create_menu(
            "pomodoro_timer_menu",
            text=_("Pomodoro timer"),
            icon=qta.icon("mdi.av-timer", color=ima.MAIN_FG_COLOR),
        )

        # Add actions to the menu
        for action in [start_timer_action, pause_timer_action, stop_timer_action]:
            self.add_item_to_menu(
                action,
                self.pomodoro_menu,
                section=PomodoroMenuSections.Main,
            )

        self.pomodoro_button = self.create_toolbutton(
            "pomodoro_timer_button",
            text=_("Pomodoro timer"),
            icon=qta.icon("mdi.av-timer", color=ima.MAIN_FG_COLOR),
        )

        self.pomodoro_button.setMenu(self.pomodoro_menu)
        self.pomodoro_button.setPopupMode(QToolButton.InstantPopup)

        # Add menu to toolbar
        self.add_item_to_toolbar(
            self.pomodoro_button,
            self.pomodoro_timer_toolbar,
            section=PomodoroToolbarSections.Controls,
        )

    def update_actions(self):
        pass

    # --- Public API
    # ------------------------------------------------------------------------

    def start_pomodoro_timer(self):
        """Start the timer."""
        self.pomodoro_timer_status.timer.start(INTERVAL)
        self.pomodoro_timer_status.pause = False

    def pause_pomodoro_timer(self):
        """Pause the timer."""
        self.pomodoro_timer_status.timer.stop()
        self.pomodoro_timer_status.pause = True

    def stop_pomodoro_timer(self):
        """Stop the timer."""
        self.pomodoro_timer_status.timer.stop()
        self.pomodoro_timer_status.pause = True
        self.pomodoro_timer_status.countdown = self.pomodoro_timer_status.pomodoro_limit



# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------------
# Copyright © 2021, Francisco Palm
#
# Licensed under the terms of the MIT license
# ----------------------------------------------------------------------------
"""
Spyder Pomodoro Timer Preferences Page.
"""
from qtpy.QtWidgets import QGridLayout, QGroupBox, QVBoxLayout
from spyder.api.preferences import PluginConfigPage
from spyder.api.translations import get_translation

from spyder_pomodoro_timer.spyder.config import POMODORO_DEFAULT

_ = get_translation("spyder_pomodoro_timer.spyder")


class SpyderPomodoroTimerConfigPage(PluginConfigPage):

    # --- PluginConfigPage API
    # ------------------------------------------------------------------------
    def setup_page(self):
        limits_group = QGroupBox(_("Time limits"))
        pomodoro_spin = self.create_spinbox(
            _("Pomodoro timer limit"),
            _("min"),
            "pomodoro_limit",
            default=POMODORO_DEFAULT,
            min_=5,
            max_=100,
            step=1,
        )

        pt_limits_layout = QGridLayout()
        pt_limits_layout.addWidget(pomodoro_spin.plabel, 0, 0)
        pt_limits_layout.addWidget(pomodoro_spin.spinbox, 0, 1)
        pt_limits_layout.addWidget(pomodoro_spin.slabel, 0, 2)
        pt_limits_layout.setColumnStretch(1, 100)
        limits_group.setLayout(pt_limits_layout)

        vlayout = QVBoxLayout()
        vlayout.addWidget(limits_group)
        vlayout.addStretch(1)
        self.setLayout(vlayout)



# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------------
# Copyright © 2021, Francisco Palm
#
# Licensed under the terms of the MIT license
# ----------------------------------------------------------------------------
"""
Spyder Pomodoro Timer Plugin.
"""

# Third-party imports
import qtawesome as qta

# Spyder imports
from spyder.api.plugin_registration.decorators import on_plugin_available
from spyder.api.plugins import Plugins, SpyderPluginV2
from spyder.api.translations import get_translation
from spyder.utils.icon_manager import ima

# Local imports
from spyder_pomodoro_timer.spyder.confpage import SpyderPomodoroTimerConfigPage
from spyder_pomodoro_timer.spyder.container import SpyderPomodoroTimerContainer
from spyder_pomodoro_timer.spyder.config import CONF_DEFAULTS, CONF_VERSION


_ = get_translation("spyder_pomodoro_timer.spyder")


class SpyderPomodoroTimer(SpyderPluginV2):
    """
    Spyder Pomodoro Timer plugin.
    """

    NAME = "spyder_pomodoro_timer"
    REQUIRES = [Plugins.Preferences, Plugins.StatusBar, Plugins.Toolbar]
    OPTIONAL = []
    CONTAINER_CLASS = SpyderPomodoroTimerContainer
    CONF_SECTION = NAME
    CONF_WIDGET_CLASS = SpyderPomodoroTimerConfigPage
    CONF_DEFAULTS = CONF_DEFAULTS
    CONF_VERSION = CONF_VERSION

    # --- Signals

    # --- SpyderPluginV2 API
    # ------------------------------------------------------------------------
    def get_name(self):
        return _("Spyder Pomodoro Timer")

    def get_description(self):
        return _("A very simple pomodoro timer that shows in the status bar.")

    def get_icon(self):
        return qta.icon("mdi.av-timer", color=ima.MAIN_FG_COLOR)

    def on_initialize(self):
        print("SpyderPomodoroTimer registered!")

    @on_plugin_available(plugin=Plugins.Preferences)
    def on_preferences_available(self):
        preferences = self.get_plugin(Plugins.Preferences)
        preferences.register_plugin_preferences(self)

    @on_plugin_available(plugin=Plugins.StatusBar)
    def on_statusbar_available(self):
        statusbar = self.get_plugin(Plugins.StatusBar)
        if statusbar:
            statusbar.add_status_widget(self.pomodoro_timer_status)

    @on_plugin_available(plugin=Plugins.Toolbar)
    def on_toolbar_available(self):
        container = self.get_container()
        toolbar = self.get_plugin(Plugins.Toolbar)
        toolbar.add_application_toolbar(container.pomodoro_timer_toolbar)

    def check_compatibility(self):
        valid = True
        message = ""  # Note: Remember to use _("") to localize the string
        return valid, message

    def on_close(self, cancellable=True):
        return True

    # --- Public API
    # ------------------------------------------------------------------------

    @property
    def pomodoro_timer_status(self):
        container = self.get_container()
        return container.pomodoro_timer_status



# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------------
# Copyright © 2021, Francisco Palm
#
# Licensed under the terms of the MIT license
# ----------------------------------------------------------------------------
"""
Spyder Pomodoro Timer API.
"""

class PomodoroToolbarActions:
    Start = 'start_timer'
    Pause = 'pause_timer'
    Stop = 'stop_timer'


class PomodoroToolbarSections:
    Controls = "pomodoro_timer"

class PomodoroMenuSections:
    Main = "main_section"



# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------------
# Copyright © 2021, Francisco Palm
#
# Licensed under the terms of the MIT license
# ----------------------------------------------------------------------------
"""
Spyder Pomodoro Timer Main Widget.
"""

# Third party imports
import qtawesome as qta


# Spyder imports
from spyder.api.config.decorators import on_conf_change
from spyder.api.translations import get_translation

from spyder.api.widgets.status import BaseTimerStatus
from spyder.utils.icon_manager import ima
from spyder.api.widgets.toolbars import ApplicationToolbar

# Local imports
from spyder_pomodoro_timer.spyder.config import (
    CONF_SECTION,
    CONF_DEFAULTS,
    CONF_VERSION,
)


# Localization
_ = get_translation("spyder_pomodoro_timer.spyder")

# --- Constants
# ------ Time limits by default

INTERVAL = 1000

# ---- Widgets
# ----------------------------------------------------------------------------


class PomodoroTimerToolbar(ApplicationToolbar):
    """Toolbar to add buttons to control our timer."""

    ID = "pomodoro_timer_toolbar"


class PomodoroTimerStatus(BaseTimerStatus):
    """Status bar widget to display the pomodoro timer"""

    ID = "pomodoro_timer_status"

    CONF_SECTION = CONF_SECTION
    CONF_DEFAULTS = CONF_DEFAULTS
    CONF_VERSION = CONF_VERSION

    def __init__(self, parent):
        super().__init__(parent)
        self.value = "25:00"

        # Actual time limits
        self.pomodoro_limit = self.get_conf(
            "pomodoro_limit"
        )
        self.countdown = self.pomodoro_limit

        self._interval = INTERVAL
        self.timer.timeout.connect(self.update_timer)
        self.timer.start(self._interval)

        self.pause = True

    def get_tooltip(self):
        """Override api method."""

        return "I am the Pomodoro timer!"

    def get_icon(self):
        return qta.icon("mdi.av-timer", color=ima.MAIN_FG_COLOR)

    # ---- BaseTimerStatus API
    def get_value(self):
        """Get current time of the timer"""

        return self.value

    def display_time(self):
        """Calculate the time that should be displayed."""

        minutes = int((self.countdown / (1000 * 60)) % 60)
        seconds = int((self.countdown / 1000) % 60)
        return f"{minutes:02d}:{seconds:02d}"

    def update_timer(self):
        """Updates the timer and the current widget. Also, update the
        task counter if a task is set."""

        if self.countdown > 0 and not self.pause:
            # Update the current timer by decreasing the current running time by one second
            self.countdown -= INTERVAL
            self.value = self.display_time()

    @on_conf_change(option="pomodoro_limit")
    def set_pomodoro_limit(self, value):
        self.pomodoro_limit = int(value) * 1000 * 60
        self.countdown = self.pomodoro_limit
        self.value = self.display_time()



# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------------
# Copyright © 2021, Francisco Palm
#
# Licensed under the terms of the MIT license
# ----------------------------------------------------------------------------
"""
Spyder Pomodoro Timer setup.
"""
from setuptools import find_packages
from setuptools import setup

from spyder_pomodoro_timer import __version__

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    # See: https://setuptools.readthedocs.io/en/latest/setuptools.html
    name="spyder-pomodoro-timer",
    version=__version__,
    author="Francisco Palm",
    author_email="fpalm@qu4nt.com",
    description="A very simple pomodoro timer that shows in the status bar.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="MIT license",
    url="https://github.com/map0logo/spyder-pomodoro-timer",
    python_requires='>= 3.7',
    install_requires=[
        "qtpy",
        "qtawesome",
        "spyder>=5.0.1",
    ],
    packages=find_packages(),
    entry_points={
        "spyder.plugins": [
            "spyder_pomodoro_timer = spyder_pomodoro_timer.spyder.plugin:SpyderPomodoroTimer"
        ],
    },
    classifiers=[
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Scientific/Engineering",
        "Topic :: Utilities"
    ],
)



# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""Spyder terminal default configuration."""

# --- Constants
# ------ Time limits by default

POMODORO_DEFAULT = 25 * 60 * 1000  # 25 mins in milliseconds

CONF_SECTION = "spyder_pomodoro_timer"

CONF_DEFAULTS = [
    (
        CONF_SECTION,
        {
            "pomodoro_limit": POMODORO_DEFAULT / (60 * 1000),
        },
    ),
    ("shortcuts", {"pomodoro-timer start/pause": "Ctrl+Alt+Shift+P"}),
]

# IMPORTANT NOTES:
# 1. If you want to *change* the default value of a current option, you need to
#    do a MINOR update in config version, e.g. from 1.0.0 to 1.1.0
# 2. If you want to *remove* options that are no longer needed in our codebase,
#    or if you want to *rename* options, then you need to do a MAJOR update in
#    version, e.g. from 1.0.0 to 2.0.0
# 3. You don't need to touch this value if you're just adding a new option
CONF_VERSION = "1.0.0"
